@@ -66,6 +66,17 @@ PooledRequest <- R6Class(
 
       private$req_prep <- req_prepare(req)
       private$handle <- req_handle(private$req_prep)
+      # Note: we need to do this before we call handle_preflight() so that
+      # request signing works correctly with the added headers.
+      #
+      # TODO: Support resend_count.
+      private$req_prep <- req_with_span(
+        private$req_prep,
+        # Pooled request spans should not become the active span; we want
+        # subsequent requests to be siblings rather than parents.
+        activate = FALSE
+      )
+      handle_preflight(private$req_prep, private$handle)
 
       curl::multi_add(
         handle = private$handle,
@@ -83,6 +94,7 @@ PooledRequest <- R6Class(
       if (!is.null(private$handle)) {
         curl::multi_cancel(private$handle)
       }
+      req_record_span_status(private$req_prep)
     }
   ),
   private = list(
@@ -114,6 +126,7 @@ PooledRequest <- R6Class(
       }
 
       resp <- create_response(self$req, curl_data, body)
+      req_record_span_status(private$req_prep, resp)
       resp <- cache_post_fetch(self$req, resp, path = private$path)
       private$handle_response(resp, self$req)
     },
@@ -136,6 +149,7 @@ PooledRequest <- R6Class(
       curl_error <- error_cnd(message = msg, class = error_class, call = NULL)
       error <- curl_cnd(curl_error, call = private$error_call)
       error$request <- self$req
+      req_record_span_status(private$req_prep, error)
       private$on_error(error)
     }
   )
