@@ -81,10 +81,16 @@ req_perform_connection <- function(
     retry_check_breaker(req, tries)
     sys_sleep(delay, "for retry backoff")
 
-    if (!is.null(resp)) {
+    if (is_response(resp)) {
       close(resp)
     }
-    resp <- req_perform_connection1(req, handle, blocking = blocking)
+    resp <- req_perform_connection1(
+      req,
+      req_prep,
+      handle,
+      blocking = blocking,
+      resend_count = tries + 1L
+    )
 
     if (retry_is_transient(req, resp)) {
       tries <- tries + 1
@@ -95,7 +101,7 @@ req_perform_connection <- function(
       break
     }
   }
-  req_completed(req)
+  req_completed(req_prep)
 
   if (!is_error(resp) && error_is_error(req, resp)) {
     # Read full body if there's an error
@@ -135,27 +141,37 @@ req_verbosity_connection <- function(
   req
 }
 
-req_perform_connection1 <- function(req, handle, blocking = TRUE) {
+req_perform_connection1 <- function(
+  req,
+  req_prep,
+  handle,
+  blocking = TRUE,
+  resend_count = 0
+) {
   the$last_request <- req
   the$last_response <- NULL
   signal(class = "httr2_perform_connection")
+  # Note: we need to do this before we call handle_preflight() so that request
+  # signing works correctly with the added headers.
+  req_prep <- req_with_span(req_prep, resend_count = resend_count)
+  handle_preflight(req_prep, handle)
 
   err <- capture_curl_error({
     conn <- curl::curl(req$url, handle = handle)
     # Must open the stream in order to initiate the connection
-    withCallingHandlers(
-      open(conn, "rbf", blocking = blocking),
-      warning = \(cnd) tryInvokeRestart("muffleWarning"),
-      error = \(cnd) close(conn)
-    )
+    suppressWarnings(open(conn, "rbf", blocking = blocking))
     body <- StreamingBody$new(conn)
   })
   if (is_error(err)) {
+    req_record_span_status(req, err)
+    close(conn)
     return(err)
   }
 
   curl_data <- curl::handle_data(handle)
-  create_response(req, curl_data, body)
+  resp <- create_response(req, curl_data, body)
+  req_record_span_status(req, resp)
+  resp
 }
 
 # Make open mockable
