@@ -13,7 +13,7 @@
 #' check with `check_type = FALSE`. These two functions also cache the parsed
 #' object so the second and subsequent calls are low-cost.
 #'
-#' @param resp A response object.
+#' @inheritParams resp_headers
 #' @returns
 #' * `resp_body_raw()` returns a raw vector.
 #' * `resp_body_string()` returns a string.
@@ -36,11 +36,14 @@ resp_body_raw <- function(resp) {
 
   if (!resp_has_body(resp)) {
     cli::cli_abort("Can't retrieve empty body.")
-  } else if (is_path(resp$body)) {
-    readBin(resp$body, "raw", file.size(resp$body))
-  } else {
-    resp$body
   }
+
+  switch(
+    resp_body_type(resp),
+    disk = readBin(resp$body, "raw", file.size(resp$body)),
+    memory = resp$body,
+    stream = resp$body$read_all()
+  )
 }
 
 #' @rdname resp_body_raw
@@ -48,10 +51,21 @@ resp_body_raw <- function(resp) {
 resp_has_body <- function(resp) {
   check_response(resp)
 
+  switch(
+    resp_body_type(resp),
+    disk = file.size(resp$body) > 0,
+    memory = length(resp$body) > 0,
+    stream = resp$body$is_open()
+  )
+}
+
+resp_body_type <- function(resp) {
   if (is_path(resp$body)) {
-    file.size(resp$body) > 0
+    "disk"
+  } else if (inherits(resp$body, "StreamingBody")) {
+    "stream"
   } else {
-    length(resp$body) > 0
+    "memory"
   }
 }
 
@@ -77,7 +91,12 @@ resp_body_string <- function(resp, encoding = NULL) {
 #'   [xml2::read_xml()] respectively.
 #' @rdname resp_body_raw
 #' @export
-resp_body_json <- function(resp, check_type = TRUE, simplifyVector = FALSE, ...) {
+resp_body_json <- function(
+  resp,
+  check_type = TRUE,
+  simplifyVector = FALSE,
+  ...
+) {
   check_response(resp)
   check_installed("jsonlite")
 
@@ -94,7 +113,11 @@ resp_body_json <- function(resp, check_type = TRUE, simplifyVector = FALSE, ...)
   )
 
   text <- resp_body_string(resp, "UTF-8")
-  resp$cache[[key]] <- jsonlite::fromJSON(text, simplifyVector = simplifyVector, ...)
+  resp$cache[[key]] <- jsonlite::fromJSON(
+    text,
+    simplifyVector = simplifyVector,
+    ...
+  )
   resp$cache[[key]]
 }
 
@@ -109,7 +132,8 @@ resp_body_html <- function(resp, check_type = TRUE, ...) {
     check_type = check_type
   )
 
-  xml2::read_html(resp$body, ...)
+  body <- resp_body_raw(resp)
+  xml2::read_html(body, ...)
 }
 
 #' @rdname resp_body_raw
@@ -130,7 +154,8 @@ resp_body_xml <- function(resp, check_type = TRUE, ...) {
     check_type = check_type
   )
 
-  resp$cache[[key]] <- xml2::read_xml(resp$body, ...)
+  body <- resp_body_raw(resp)
+  resp$cache[[key]] <- xml2::read_xml(body, ...)
   resp$cache[[key]]
 }
 
