@@ -68,24 +68,13 @@ test_that("adds port to localhost url", {
   expect_false(is.null(url_parse(redirect$uri)$port))
 })
 
-test_that("old args are deprecated", {
+test_that("urls left as is if not changes needed", {
   # Allow tests to run when is_hosted_session() is TRUE.
   local_mocked_bindings(is_hosted_session = function() FALSE)
 
-  expect_snapshot(
-    redirect <- normalize_redirect_uri("http://localhost", port = 1234)
-  )
-  expect_equal(redirect$uri, "http://localhost:1234/")
-
-  expect_snapshot(
-    redirect <- normalize_redirect_uri("http://x.com", host_name = "y.com")
-  )
-  expect_equal(redirect$uri, "http://y.com/")
-
-  expect_snapshot(
-    redirect <- normalize_redirect_uri("http://x.com", host_ip = "y.com")
-  )
-
+  original_uri <- "http://localhost:8080"
+  normalized_uri <- normalize_redirect_uri(original_uri)
+  expect_equal(normalized_uri$uri, original_uri)
 })
 
 # ouath_flow_auth_code_parse ----------------------------------------------
@@ -125,20 +114,16 @@ test_that("external auth code sources are detected correctly", {
 
 test_that("auth codes can be retrieved from an external source", {
   skip_on_cran()
+  local_mocked_bindings(sys_sleep = function(...) {})
 
   req <- local_app_request(function(req, res) {
     # Error on first, and then respond on second
     authorized <- res$app$locals$authorized %||% FALSE
     if (!authorized) {
       res$app$locals$authorized <- TRUE
-      res$
-        set_status(404L)$
-        set_type("text/plain")$
-        send("Not found")
+      res$set_status(404L)$set_type("text/plain")$send("Not found")
     } else {
-      res$
-        set_status(200L)$
-        send_json(text = '{"code":"abc123"}')
+      res$set_status(200L)$send_json(text = '{"code":"abc123"}')
     }
   })
 
