@@ -1,5 +1,5 @@
 test_that("can extract url components from a response", {
-  req <- request_test("/get") %>% req_url_query(a = "1", b = "2")
+  req <- request_test("/get") |> req_url_query(a = "1", b = "2")
   resp <- req_perform(req)
 
   expect_equal(resp_url(resp), example_url("/get?a=1&b=2"))
